@@ -4,7 +4,6 @@ from urllib.parse import urlparse
 from .auth import Login
 from .public import (
     PublicRequest,
-    HashtagPublic,
     TopSearchesPublic,
     ProfilePublic,
 )
@@ -21,13 +20,13 @@ from .collection import Collection
 from .account import Account
 from .direct import Direct
 from .location import LocationMixin
+from .hashtag import HashtagMixin
 
 
 class Client(
     PublicRequest,
     ChallengeResolve,
     PrivateRequest,
-    HashtagPublic,
     TopSearchesPublic,
     ProfilePublic,
     Login,
@@ -45,7 +44,8 @@ class Client(
     Collection,
     Account,
     Direct,
-    LocationMixin
+    LocationMixin,
+    HashtagMixin
 ):
     proxy = None
     logger = logging.getLogger("instagrapi")
